package solution;

import java.util.Arrays;

public class Demo {

  /**
   * Sorts the input array.
   *
   * @param arr an array of integers.
   */
  public static void quicksort(int[] arr) {
    quicksort(arr, 0, arr.length);
  }

  // Pre: 0 <= left <= right <= arr.length.
  // Post: arr[left] ... arr[right-1] is sorted.
  private static void quicksort(int[] arr, int left, int right) {
    if (right - left < 2) {
      return;
    }

    int pivot = partition(arr, left, right);
    quicksort(arr, left, pivot);
    quicksort(arr, pivot + 1, right);
  }

  // Partition by taking the right-most element as pivot.
  // Pre: 0 <= left <= right <= arr.length;
  // Post: a[left] <= .. <= pivot <= .. <= a[right - 1]
  // Returns: index of pivot element in array.
  private static int partition(int[] arr, int left, int right) {
    int p = right - 1; // point to pivot
    int l = left; // left marker
    int r = p - 1; // right marker

    while (l <= r) {
      while (arr[l] <= arr[p] && l <= r) {
        l++; // move l to right as long as condition holds.
      }

      while (arr[r] >= arr[p] && r >= l) {
        r--; // move r to left as long as condition holds.
      }

      if (l < r) {
        // arr[l] and arr[r] are out of order with respect to pivot
        swap(arr, l, r);
      }
    }

    if (l != p) {
      // move the pivot to its final place
      swap(arr, l, p);
    }

    return l; // return index of item now known to be in place
  }

  // Swap arr[i] with arr[j]
  private static void swap(int[] arr, int i, int j) {
    int tmp = arr[i];
    arr[i] = arr[j];
    arr[j] = tmp;
  }

  /**
   * Demo starts here.
   *
   * @param args command-line arguments.
   */
  public static void main(String[] args) {
    int[] arr = {20, 13, 7, 71, 31, 10, 5, 50, 100};
    System.out.println("Input: " + Arrays.toString(arr));
    quicksort(arr);
    System.out.println("Output: " + Arrays.toString(arr));
  }
}